function [K scale sums] = kernelize(kernel, X, nscale)

%
% [K scale sums] = kernelize(kernel, X, nscale);
%
% Kernelize (training) data
%
% Input
% kernel    - kernel type, 'Gaussian' etc.
% X         - training data
% nscale    - scale parameter for kernel
%
% Output
% K         - kernel for training data
% scale     - scale parameter for kernel
% sums      - aux variable for training data (symmetric)
%

% (c) Copyright 2010
% Allan Aasbjerg Nielsen
% aa@space.dtu.dk, www.imm.dtu.dk/~aa
% 15 Mar 2010

if nargin<2, error('kernelize: wrong input'); end
if nargin<3, nscale = 1; end

% kernel = 'Gaussian';
% kernel = 'multiquadric';
% kernel = 'inverse multiquadric';
% kernel = 'linear, identity mapping';
% kernel = '2, second order polynomial';

[n p] = size(X);
sums = NaN;

if ~strncmp(kernel,'l',1)
    sums = sum(X.*X,2);
    K = repmat(sums',n,1);
    K = K + K' - 2*X*X'; % squared distances beetween training obs, i.e., rows of X
    scale = nscale*sum(real(sqrt(K(:))))/(n*(n-1)); % mean distance between obs
end
if strncmp(kernel,'G',1)
    sigma2 = 2*scale^2;
    K = exp(-K/sigma2); % Gaussian
elseif strncmp(kernel,'m',1) 
    K  = sqrt(K + scale^2); % multiquadric
elseif strncmp(kernel,'i',1) 
    K  = 1./sqrt(K + scale^2); % inverse multiquadric
elseif strncmp(kernel,'l',1)
    K  = X*X'; % identity mapping, linear kernel
    scale = NaN;
else
    error('wrong kernel specification')
end
