function K = kernelize2(kernel, Xtrain, Xtest, scale, Xtrainsum)

%
% K = kernelize2(kernel, Xtrain, Xtest, scale, Xtrainsum);
%
% Kernelize new (test) data with training data
%
% Input
% kernel    - kernel type, 'Gaussian' etc.
% Xtrain    - training data
% Xtest     - new test data to be kernelized with training data
% scale     - scale parameter for kernel
% Xtrainsum - aux variable for training data
%             (row or columns sum for symmetric kernel matrix)
%
% Output
% K         - kernel for test data with training data
%

% (c) Copyright 2010
% Allan Aasbjerg Nielsen
% aa@space.dtu.dk, www.imm.dtu.dk/~aa
% 14 Mar 2010

if nargin<3, error('kernelize2: wrong input'); end
if nargin<4, scale = 1; end

% kernel = 'Gaussian';
% kernel = 'multiquadric';
% kernel = 'inverse multiquadric';
% kernel = 'linear, identity mapping';
% kernel = '2, second order polynomial';

[n p] = size(Xtrain);
[ntest ptest] = size(Xtest);
if p~=ptest, error('Xtrain and Xtest must have same #variables'); end

if ~strncmp(kernel,'l',1)
    if nargin<5, Xtrainsum = sum(Xtrain.*Xtrain,2); end
    sums0 = sum(Xtest.*Xtest,2);
    H0 = repmat(sums0',n,1);
    Hi = repmat(Xtrainsum',ntest,1); % Hi = repmat(sums',ntest,1);
    K = H0 + Hi' - 2*Xtrain*Xtest';
    clear H0 Hi
end
if strncmp(kernel,'G',1)
    sigma2 = 2*scale^2;
    K = exp(-K/sigma2); % Gaussian
elseif strncmp(kernel,'m',1) 
    K  = sqrt(K  + scale^2); % multiquadric
elseif strncmp(kernel,'i',1) 
    K  = 1./sqrt(K  + scale^2); % inverse multiquadric
elseif strncmp(kernel,'l',1)
    K  = Xtrain*Xtest'; % identity mapping, linear kernel
else
    error('wrong kernel specification')
end
