function [K Kn scale sums] = kernelizemnf(kernel, X, Xn1, Xn2, Xn3, Xn4, Xn6, Xn7, Xn8, Xn9, nscale)

%
% [K Kn scale sums] = kernelizemnf(kernel, X, Xn1, Xn2, Xn3, Xn4, Xn6, Xn7, Xn8, Xn9, nscale);
%
% Kernelize (training) data and eight shifted versions and pool
% (typically for kernel MNF analysis)
%
% Input
% kernel    - kernel type, 'Gaussian' etc.
% X         - training data
% Xn1       - training data, shifted version
% Xn2       - training data, shifted version
% Xn3       - training data, shifted version
% Xn4       - training data, shifted version
% Xn6       - training data, shifted version (Xn5=X)
% Xn7       - training data, shifted version
% Xn8       - training data, shifted version
% Xn9       - training data, shifted version
% nscale    - scale parameter for kernel
%
% Output
% K         - kernel for training data
% Kn        - kernel for shifted training data
% scale     - scale parameter for kernel
% sums      - aux variable for training data (symmetric)
%

% (c) Copyright 2010
% Allan Aasbjerg Nielsen
% aa@space.dtu.dk, www.imm.dtu.dk/~aa
% 15 Mar 2010

if nargin<10, error('kernelizemnf: wrong input'); end
if nargin<11, nscale = 1; end

% kernel = 'Gaussian';
% kernel = 'multiquadric';
% kernel = 'inverse multiquadric';
% kernel = 'spline, thin-plate';
% kernel = 'linear, identity mapping';
% kernel = '2, second order polynomial';
if ~strncmp(kernel,'G',1)
    error('So far only the Gaussian kernel works')
end

[n  p ] = size(X);
[ny py] = size(Xn1);
if p~=py, error('X and Xn1 must have same #variables'); end
[ny py] = size(Xn2);
if p~=py, error('X and Xn2 must have same #variables'); end
[ny py] = size(Xn3);
if p~=py, error('X and Xn3 must have same #variables'); end
[ny py] = size(Xn4);
if p~=py, error('X and Xn4 must have same #variables'); end
[ny py] = size(Xn6);
if p~=py, error('X and Xn6 must have same #variables'); end
[ny py] = size(Xn7);
if p~=py, error('X and Xn7 must have same #variables'); end
[ny py] = size(Xn8);
if p~=py, error('X and Xn8 must have same #variables'); end
[ny py] = size(Xn9);
if p~=py, error('X and Xn9 must have same #variables'); end
sums = NaN;

if strncmp(kernel,'G',1)
%
    sums = sum(X.*X,2);
    Hi = repmat(sums',n,1);
    K = Hi;
    K = K + K' - 2*X*X'; % squared distances beetween training obs, i.e., rows of X
    scale = nscale*sum(real(sqrt(K(:))))/(n*(n-1)); % mean distance between obs
    scalen = scale;
    sigma2 = 2*scale^2;
    sigman2 = 2*scalen^2;
    K = exp(-K/sigma2); % Gaussian
%
    sums0 = sum(Xn1.*Xn1,2);
    H0 = repmat(sums0',n,1);
%     Hi = repmat(sums',n,1);
    D  = H0 + Hi' - 2*X*Xn1' ; % squared distances beween X and Xn1
    Kn = -exp(-D/sigman2);
    clear Xn1
    sums0 = sum(Xn2.*Xn2,2);
    H0 = repmat(sums0',n,1);
    D = H0 + Hi' - 2*X*Xn2'; % squared distances beween X and Xn2
    Kn = Kn + 2*exp(-D/sigman2);
    clear Xn2
    sums0 = sum(Xn3.*Xn3,2);
    H0 = repmat(sums0',n,1);
    D = H0 + Hi' - 2*X*Xn3' ; % squared distances beween X and Xn3
    Kn = Kn - exp(-D/sigman2);
    clear Xn3
    sums0 = sum(Xn4.*Xn4,2);
    H0 = repmat(sums0',n,1);
    D = H0 + Hi' - 2*X*Xn4' ; % squared distances beween X and Xn4
    Kn = Kn + 2*exp(-D/sigman2);
    clear Xn4
    sums0 = sum(Xn6.*Xn6,2);
    H0 = repmat(sums0',n,1);
    D = H0 + Hi' - 2*X*Xn6' ; % squared distances beween X and Xn6
    Kn = Kn + 2*exp(-D/sigman2);
    clear Xn6
    sums0 = sum(Xn7.*Xn7,2);
    H0 = repmat(sums0',n,1);
    D = H0 + Hi' - 2*X*Xn7' ; % squared distances beween X and Xn7
    Kn = Kn - exp(-D/sigman2);
    clear Xn7
    sums0 = sum(Xn8.*Xn8,2);
    H0 = repmat(sums0',n,1);
    D = H0 + Hi' - 2*X*Xn8' ; % squared distances beween X and Xn8
    Kn = Kn + 2*exp(-D/sigman2);
    clear Xn8
    sums0 = sum(Xn9.*Xn9,2);
    H0 = repmat(sums0',n,1);
    D = H0 + Hi' - 2*X*Xn9' ; % squared distances beween X and Xn9
    Kn = Kn - exp(-D/sigman2);
    clear Xn9 X
	clear D H0 Hi sums0
% end
% elseif strncmp(kernel,'m',1) 
%     K   = sqrt(K   + scale^2); % multiquadric
%     Kn  = sqrt(Kn  + scalen^2);
%     Kn2 = sqrt(Kn2 + scalen^2);
%     Kn3 = sqrt(Kn3 + scalen^2);
%     Kn4 = sqrt(Kn4 + scalen^2);
%     Kn6 = sqrt(Kn6 + scalen^2);
%     Kn7 = sqrt(Kn7 + scalen^2);
%     Kn8 = sqrt(Kn8 + scalen^2);
%     Kn9 = sqrt(Kn9 + scalen^2);
% elseif strncmp(kernel,'i',1) 
%     K   = 1./sqrt(K   + scale^2); % inverse multiquadric
%     Kn  = 1./sqrt(Kn  + scalen^2);
%     Kn2 = 1./sqrt(Kn2 + scalen^2);
%     Kn3 = 1./sqrt(Kn3 + scalen^2);
%     Kn4 = 1./sqrt(Kn4 + scalen^2);
%     Kn6 = 1./sqrt(Kn6 + scalen^2);
%     Kn7 = 1./sqrt(Kn7 + scalen^2);
%     Kn8 = 1./sqrt(Kn8 + scalen^2);
%     Kn9 = 1./sqrt(Kn9 + scalen^2);
% elseif strncmp(kernel,'l',1)
%     K   = X *X'  ; % identity mapping, linear kernel
%     Kn  = X *Xn1';
%     Kn2 = X *Xn2';
%     Kn3 = X *Xn3';
%     Kn5 = X *Xn4';
%     Kn6 = X *Xn6';
%     Kn7 = X *Xn7';
%     Kn8 = X *Xn8';
%     Kn9 = X *Xn9';
%     scale = NaN;
else
    error('wrong kernel specification')
end
% mean in 3x3 window
%     Kn = K - (Kn + Kn2 + Kn3 + Kn4 + K + Kn6 + Kn7 + Kn8 + Kn9)/9;
% quadratic in 3x3 window
% %     Kn = (Kn-2*Kn2+Kn3-2*Kn4+4*K-2*Kn6+Kn7-2*Kn8+Kn9)/9;
% Kn = K - (-Kn+2*Kn2-Kn3+2*Kn4+5*K+2*Kn6-Kn7+2*Kn8-Kn9)/9;
% clear Kn2 Kn3 Kn4 Kn6 Kn7 Kn8 Kn9
Kn = 4*K/9 - Kn/9;
